@@ -20,83 +20,9 @@
 #include "avifmt.h"
 #include <gdk/gdkkeysyms.h>
 #include "misc/fileio.h"
-#include "misc/misc.h"
 #include "misc/profile.h"
 
 
-/******************************************************************************
-*                                                                             *
-* ja:ICM内部処理                                                              *
-*                                                                             *
-******************************************************************************/
-/*  ja:4文字コード→文字列
-    handler,4文字コード
-        RET,文字列                                                          */
-/*static gchar *
-icm_handler_to_string (const guint handler)
-{
-  gchar c, *text = NULL, *temp0, *temp1;
-  gint i;
-
-  for (i = 24; i >= 0; i -= 8)
-    {
-      c = (handler >> i) & 0xff;
-      temp0 = g_strdup_printf (g_ascii_isalnum (c) ? "%c" : "%%%02x",
-                                                                    (guint8)c);
-      temp1 = g_strconcat (temp0, text, NULL);
-      g_free (temp0);
-      g_free (text);
-      text = temp1;
-    }
-  return text;
-}*/
-
-
-#if defined (G_OS_WIN32) || defined (W32CODECDIR)
-/*  ja:文字列→4文字コード
-    text,文字列
-     RET,4文字コード                                                        */
-static guint32
-icm_string_to_handler (const gchar *text)
-{
-  gint i, j = 0, k;
-  guint32 handler = 0;
-
-  for (i = 0; i < 4; i++)
-    {
-      handler >>= 8;
-      if (text[j] == '%')
-        {
-          j++;
-          for (k = 28; k >= 24; k -= 4)
-            {
-              if ('0' <= text[j] && text[j] <= '9')
-                handler |= (text[j] - '0') << k;
-              else if ('A' <= text[j] && text[j] <= 'A')
-                handler |= (text[j] - 'A') << k;
-              else if ('a' <= text[j] && text[j] <= 'f')
-                handler |= (text[j] - 'a')<<k;
-              else
-                break;
-              j++;
-            }
-          if (k>=24)
-            return 0;
-        }
-      else if (text[j] != '\0')
-        {
-          handler |= text[j++] << 24;
-        }
-      else
-        {
-          return 0;
-        }
-    }
-  return handler;
-}
-#endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
-
-
 /******************************************************************************
 *                                                                             *
 * ja:ICM初期化                                                                *
@@ -111,45 +37,36 @@ icm_init_callback (gpointer key,
                    gpointer value,
                    gpointer ghash)
 {
-# ifdef G_OS_WIN32
-  LPTSTR path;
-# else /* not G_OS_WIN32 */
-  gchar *path;
-# endif /* not G_OS_WIN32 */
-  guint32 handler;
-  IcmDriver *icm_driver;
+  gpointer orig_key, orig_value;
 
-  if (g_hash_table_lookup_extended ((GHashTable*)ghash, key,
-                                    (gpointer *)&handler, (gpointer *)&path))
+  if (g_hash_table_lookup_extended ((GHashTable *)ghash, key,
+                                                    &orig_key, &orig_value))
     {
+# ifdef G_OS_WIN32
+      LPTSTR path;
+# else /* not G_OS_WIN32 */
+      gchar *path;
+# endif /* not G_OS_WIN32 */
+      guint32 handler;
+      IcmDriver *icm_driver;
+
+      handler = (guint32)orig_key;
+      path = orig_value;
       icm_driver = g_hash_table_lookup (ghash_path, path);
       if (!icm_driver)
         {
           /* ja:新規ロード */
-# ifdef G_OS_WIN32
           if ((icm_driver = g_malloc0 (sizeof (IcmDriver)))
+# ifdef G_OS_WIN32
                     && (icm_driver->hLib = LoadLibrary (path))
                     && ((FARPROC)icm_driver->DriverProc
                             = GetProcAddress (icm_driver->hLib, "DriverProc")))
-            {
-              icm_driver->DriverProc (0, icm_driver->hLib, DRV_LOAD, 0, 0);
-              icm_driver->DriverProc (0, icm_driver->hLib, DRV_ENABLE, 0, 0);
-              g_hash_table_insert (ghash_path, path, icm_driver);
-            }
-          else
-            {
-              if (icm_driver->hLib)
-                FreeLibrary (icm_driver->hLib);
-              g_free (icm_driver);
-              g_free (path);
-              icm_driver = NULL;
-            }
 # else /* not G_OS_WIN32 */
-          if ((icm_driver = g_malloc0 (sizeof (IcmDriver)))
                     && (icm_driver->hLib = peimage_create())
                     && peimage_load (icm_driver->hLib, path)
                     && (icm_driver->DriverProc
                         = peimage_resolve (icm_driver->hLib, "DriverProc")))
+# endif /* not G_OS_WIN32 */
             {
               icm_driver->DriverProc (0, icm_driver->hLib, DRV_LOAD, 0, 0);
               icm_driver->DriverProc (0, icm_driver->hLib, DRV_ENABLE, 0, 0);
@@ -158,12 +75,15 @@ icm_init_callback (gpointer key,
           else
             {
               if (icm_driver->hLib)
+# ifdef G_OS_WIN32
+                FreeLibrary (icm_driver->hLib);
+# else /* not G_OS_WIN32 */
                 peimage_destroy (icm_driver->hLib);
+# endif /* not G_OS_WIN32 */
               g_free (icm_driver);
               g_free (path);
               icm_driver = NULL;
             }
-# endif /* not G_OS_WIN32 */
         }
       if (icm_driver)   /* ja:登録 */
         g_hash_table_insert (ghash_handler, (gpointer)handler, icm_driver);
@@ -302,12 +222,14 @@ icm_exit_callback (gpointer key,
                    gpointer value,
                    gpointer ghash)
 {
-  gchar *path;
-  IcmDriver *icm_driver;
+  gpointer orig_key, orig_value;
 
-  if (g_hash_table_lookup_extended ((GHashTable*)ghash, key,
-                                (gpointer *)&path, (gpointer *)&icm_driver))
+  if (g_hash_table_lookup_extended ((GHashTable *)ghash, key,
+                                                    &orig_key, &orig_value))
     {
+      IcmDriver *icm_driver;
+
+      icm_driver = orig_value;
       icm_driver->DriverProc (0, icm_driver->hLib, DRV_FREE, 0, 0);
 #ifdef G_OS_WIN32
       FreeLibrary (icm_driver->hLib);
@@ -316,7 +238,7 @@ icm_exit_callback (gpointer key,
       peimage_destroy (icm_driver->hLib);
 #endif /* W32CODECDIR */
       g_free (icm_driver);
-      g_free (path);
+      g_free (orig_key);
     }
 }
 
@@ -338,6 +260,77 @@ icm_exit (void)
 }
 
 
+/******************************************************************************
+*                                                                             *
+* ja:ICM4文字コード処理                                                       *
+*                                                                             *
+******************************************************************************/
+/*  ja:4文字コード→文字列
+    handler,4文字コード
+        RET,文字列                                                          */
+gchar *
+icm_handler_to_string (const guint32 handler)
+{
+  gchar c, *text = NULL, *temp0, *temp1;
+  gint i;
+
+  for (i = 24; i >= 0; i -= 8)
+    {
+      c = (handler >> i) & 0xff;
+      temp0 = g_strdup_printf (g_ascii_isalnum (c) ? "%c" : "%%%02x",
+                                                                    (guint8)c);
+      temp1 = g_strconcat (temp0, text, NULL);
+      g_free (temp0);
+      g_free (text);
+      text = temp1;
+    }
+  return text;
+}
+
+
+/*  ja:文字列→4文字コード
+    text,文字列
+     RET,4文字コード                                                        */
+guint32
+icm_string_to_handler (const gchar *text)
+{
+  gint i, j = 0, k;
+  guint32 handler = 0;
+
+  for (i = 0; i < 4; i++)
+    {
+      handler >>= 8;
+      if (text[j] == '%')
+        {
+          j++;
+          for (k = 28; k >= 24; k -= 4)
+            {
+              if ('0' <= text[j] && text[j] <= '9')
+                handler |= (text[j] - '0') << k;
+              else if ('A' <= text[j] && text[j] <= 'A')
+                handler |= (text[j] - 'A') << k;
+              else if ('a' <= text[j] && text[j] <= 'f')
+                handler |= (text[j] - 'a')<<k;
+              else
+                break;
+              j++;
+            }
+          if (k>=24)
+            return 0;
+        }
+      else if (text[j] != '\0')
+        {
+          handler |= text[j++] << 24;
+        }
+      else
+        {
+          return 0;
+        }
+    }
+  return handler;
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:ICM基本                                                                  *
@@ -463,8 +456,8 @@ icm_get_info (IcmObject *icm_object,
   icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
                                     icm_object->icm_driver->hLib, ICM_GETINFO,
                                             (LONG)&icinfo, sizeof (ICINFO));
-  icm_info->name = misc_widestring_to_multistring (icinfo.szName);
-  icm_info->handler = icinfo.fccHandler;
+  icm_info->name = g_utf16_to_utf8 (icinfo.szName, -1, NULL, NULL, NULL);
+  icm_info->handler = icm_get_handler (icm_object); /* icinfo.fccHandler */
   icm_info->flags = 0;
   if (icinfo.dwFlags & VIDCF_QUALITY)
     icm_info->flags = ICM_FLAG_QUALITY;
@@ -795,8 +788,8 @@ icm_compress_frames_info (IcmObject              *icm_object,
   if (icm_info.flags & ICM_FLAG_COMPRESSFRAMES)
     {
       g_memset (&iccompressframes, 0, sizeof (ICCOMPRESSFRAMES));
-      iccompressframes.lpbiOutput = icm_object->bmih_out;
-      iccompressframes.lpbiInput = icm_object->bmih_in;
+      (BitmapInfoHeader *)iccompressframes.lpbiOutput = icm_object->bmih_out;
+      (BitmapInfoHeader *)iccompressframes.lpbiInput = icm_object->bmih_in;
       iccompressframes.lFrameCount = icm_object->frame;
       iccompressframes.lQuality = icm_object->quality;
       iccompressframes.lDataRate = data_rate > 0 ? data_rate : G_MAXINT;
@@ -891,9 +884,9 @@ icm_compress (IcmObject *icm_object,
                             || !icm_object->prev || icm_object->prev_size <= 0)
     return 0;
   iccompress.dwFlags = ICCOMPRESS_KEYFRAME;
-  iccompress.lpbiOutput = icm_object->bmih_out;
+  (BitmapInfoHeader *)iccompress.lpbiOutput = icm_object->bmih_out;
   iccompress.lpOutput = output;
-  iccompress.lpbiInput = icm_object->bmih_in;
+  (BitmapInfoHeader *)iccompress.lpbiInput = icm_object->bmih_in;
   iccompress.lpInput = input;
   iccompress.lpckid = &dwID;
   iccompress.lpdwFlags = &dwFlags;
@@ -902,7 +895,7 @@ icm_compress (IcmObject *icm_object,
   iccompress.dwQuality = icm_object->quality;
   if (icm_object->frame > 0)
     {
-      iccompress.lpbiPrev = icm_object->bmih_in;
+      (BitmapInfoHeader *)iccompress.lpbiPrev = icm_object->bmih_in;
       iccompress.lpPrev = icm_object->prev;
     }
   else
@@ -1072,9 +1065,9 @@ icm_decompress (IcmObject   *icm_object,
     icdecompress.dwFlags |= ICDECOMPRESS_NULLFRAME;
   if (flags & ICM_DECOMPRESS_NOTKEYFRAME)
     icdecompress.dwFlags |= ICDECOMPRESS_NOTKEYFRAME;
-  icdecompress.lpbiInput = icm_object->bmih_in;
+  (BitmapInfoHeader *)icdecompress.lpbiInput = icm_object->bmih_in;
   icdecompress.lpInput = input;
-  icdecompress.lpbiOutput = icm_object->bmih_out;
+  (BitmapInfoHeader *)icdecompress.lpbiOutput = icm_object->bmih_out;
   icdecompress.lpOutput = output;
   icdecompress.ckid = 0;
   return icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
